#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jul 22 21:56:31 2019

@author: Anne-Cécile GAY
"""

###############################################################################
#                           IMPORT LIBRAIRIES                                 #
###############################################################################

# PANDAS
import pandas as pd
pd.set_option('display.max_colwidth', -1)

# SKLEARN
from sklearn.feature_extraction.text import CountVectorizer,TfidfVectorizer


# GENSIM
import gensim
from gensim import corpora  
from gensim.utils import simple_preprocess,tokenize

# NLTK
import nltk
from nltk.tokenize import MWETokenizer # pour gérer les mots composés
nltk.download('stopwords')
from nltk.corpus import stopwords
from nltk.stem.snowball import FrenchStemmer

import warnings
warnings.filterwarnings("ignore", 'This pattern has match groups') 


###############################################################################
#                         PRE-TRAITEMENTS DU TEXTE                            #       
###############################################################################

# OBJECTIFS : GESTION DE LA PONCTUATION, PASSAGE EN MINUSCULES, SUPPRESSION MOTS COURTS, GESTION DES MOTS COMPOSÉS

################
#  FONCTIONS   #
################

# FONCTION : PASSAGE EN MINUSCULE + SUPPRESSION DES ACCENTS + SUPPRESSION DE LA PONCTUATION         + SUPPRESSION DES CARACTÈRES NUMÉRIQUES + SUPPRESSION DES MOTS COURTS (paramétrage) #
def fct_mef_text_1(my_text,min_len):
    my_text_modif = " ".join(simple_preprocess(str(my_text),deacc='True',min_len=min_len))
    return(my_text_modif)
    
# FONCTION : TOKENISER UN TEXTE EN INTEGRANT DES MOTS COMPOSÉS #
def fct_tokenize_mwe(my_text,my_list_mwe) : # Multi Words EXpression
    tokenizer = MWETokenizer(my_list_mwe, separator='_')
    my_text_modif = tokenizer.tokenize(str(my_text).split())
    return(my_text_modif)

# FONCTION : NORMALISATION DU TEXTE ET INTEGRATION DE MOTS COMPOSÉS #
def fct_tokenize_mef_mwe(my_text,min_len,my_list_mwe):
    #on transforme la liste de strings en tuples en la normalisant au préalable
    list_tuple = [tuple(fct_mef_text_1(mwe,min_len).split(" ")) for mwe in my_list_mwe]
    # on normalise et tokenize le texte
    my_text_modif = fct_tokenize_mwe(fct_mef_text_1(my_text,min_len),list_tuple)
    return(my_text_modif)
    
# FONCTION : df avec mots et fréquence du vocabulaire du corpus
# fit_transform de CountVectorizer permet la création de la matrice termes-documents
# Création de la liste "vocabulary"
# de la matrice terme/fréquence (X)
# et du corpus de mots (df "corpus")
def fct_df_vocab(df, col_name) : # Liste du vocabulaire
    #vectorizer = TfidfVectorizer()
    vectorizer = CountVectorizer()
    X=vectorizer.fit_transform(df[col_name])
    vocabulary = vectorizer.get_feature_names()
    X_df = pd.DataFrame({'word_id': list(X.nonzero()[1]),
                     'doc_id': list(X.nonzero()[0]),
                     'occurrence': list(X.data)})
    corpus = X_df.groupby('word_id')["occurrence"].sum().to_frame()    
    corpus["word"] = [vocabulary[i] for i in corpus.index]
    return corpus

# FONCTION : liste des n mots les plus fréquents
def fct_df_mots_freq(df,col_name, nb_mostfreq):
    df_corpus = fct_df_vocab(df, col_name)
    words_freq = df_corpus.sort_values(by = 'occurrence',ascending = False).iloc[:nb_mostfreq]
    return(words_freq)

# OK
# FONCTION : suppression des stopwords du corpus
# On supprime les stopwords de NLTK + une liste de mots à paramétrer 
def fct_supp_stopwords(my_text, list_new_sw) :
    sw = list(stopwords.words('french'))+list_new_sw
    return " ".join([w for w in str(my_text).split() if not w in sw])

# OK
# FONCTION : racinisation d'un texte
def fct_stem(MyText):
    stemmer = FrenchStemmer()
    return " ".join([stemmer.stem(w) for w in MyText.split()])

# OK
# FONCTION : PRE-TRAITEMENT DU TEXTE (normalisation, suppression stopwords)
def fct_pretraitement(df,APP_VAR,min_len_w,list_mwe,list_sw):
    # AJOUT D'UNE COLONNE AVEC VERBATIMS TOKENISÉS (avec intégration mots composés)
    print('**********   tokenisation des verbatim...   *****')
    df['VbtToken'] = df[APP_VAR].apply(lambda text : fct_tokenize_mef_mwe(text,min_len_w,list_mwe)).apply(lambda x: ' '.join(x))
    # AJOUT D'UNE COLONNE AVEC SUPPRESSION DES STOPWORDS
    print('**********   suppression des stop-words...   *****')
    df['VbtTokenNosw'] = df['VbtToken'].apply(lambda text : fct_supp_stopwords(str(text),list_sw))
    # AJOUT D'UNE COLONNE AVEC VERBATIM RACINISE
    print('**********   racinisation des termes...   *****')
    df['VbtTokenNoswStem'] = df['VbtTokenNosw'].apply(lambda text : fct_stem(str(text))) 
    return(df)
    

# FONCTION : filtre du df sur une liste de notes
def fct_df_filtre_nps(df,APP_GRADE,list_notes_nps):
    df_nps = df[df[APP_GRADE].isin(list_notes_nps)]
    return df_nps    

# FONCTION : filtre du df sur les verbatims contenant le mot word
def fct_filtre_word(df,word):  
    df_filtre_word = df.loc[df["VbtTokenNosw"].str.contains("(\s|^)"+word+"(\s|$)",regex=True).fillna(False)]    
    return(df_filtre_word)


# NOTE MOYENNE DES COMMENTAIRES CONTENANT LE MOT WORD
def fct_note_moy_word(df,APP_GRADE,word):  
    note_moy_word = df.loc[df["VbtTokenNosw"].str.contains("(\s|^)"+word+"(\s|$)",regex=True).fillna(False),APP_GRADE].mean()     
    return(note_moy_word)


# NOMBRE DE DETRACTEURS, NEUTRES, PROMOTEURS PAR MOT
def nb_statut_word(df,APP_GRADE,word,statut,dict_statut_note) :
    df_statut_word = df.loc[df["VbtTokenNosw"].str.contains("(\s|^)"+word+"(\s|$)",regex=True).fillna(False),APP_GRADE]
    nb_statut_word = len(df_statut_word.loc[df_statut_word.isin(dict_statut_note[statut])])
    return(nb_statut_word)


###############################################################################
#                         MODELISATION THÉMATIQUE                             #       
###############################################################################

def init_dico_corpusbow(df,var):
    df['lst_vbt_for_lda'] = df[var].apply(lambda text : str(text).split())
    # Initialisation du dictionnaire
    dictionary = corpora.Dictionary(df['lst_vbt_for_lda']) 
    # Corpus BOW
    corpus_bow = [dictionary.doc2bow(text) for text in df['lst_vbt_for_lda'] if text != ['nan']]
    return dictionary, corpus_bow

def model_lda(dictionary, corpus_bow, num_topics, passes) :
    ldamodel = gensim.models.ldamodel.LdaModel(corpus_bow, num_topics = num_topics, id2word=dictionary, passes=passes,random_state=999)
    topics = ldamodel.print_topics(num_words=10)
    return topics, ldamodel
    for topic in topics:
        print(topic)
    
from gensim.models.coherencemodel import CoherenceModel
def compare_models_lda(df, dictionary, corpus_bow, min_num_topics, max_num_topics, passes) :
    list_models = []
    list_coherences = []
    for num_topics in range(min_num_topics,max_num_topics,1) :
        ldamodel = gensim.models.ldamodel.LdaModel(corpus_bow, num_topics = num_topics, id2word=dictionary, passes=passes,random_state=999)
        coherencemodel = CoherenceModel(model=ldamodel, texts=df['lst_vbt_for_lda'], dictionary=dictionary, coherence='c_v')
        list_models.append(ldamodel)
        list_coherences.append(coherencemodel.get_coherence())
    return(list_models,list_coherences)
        

# FONCTION : retourne la liste des topics et les probas associées d'un texte  
def proba_topic(mytext,mylda,mydict) :
    new_doc = list(tokenize(mytext, deacc=True, lowercase=True))
    new_doc_bow = mydict.doc2bow(new_doc)
    list_topic_text = mylda[new_doc_bow] 
    return list_topic_text

# FONCTION : TOPIC PRINCIPAL - renvoie le couple (topic,proba) #
def max_proba_topic(mytext,mylda,mydict) :
    list_topic_text = proba_topic(mytext,mylda,mydict)
    max_proba = max(list_topic_text,key=lambda mytuple:mytuple[1]) # A chaque tuple, le max se fait sur la 2ème colonne
    return max_proba
