#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Aug 12 23:05:43 2019

@author: Anne-Cécile GAY
"""

from bokeh.core.properties import value
from bokeh.io import curdoc, show
from bokeh.layouts import layout,row, column, Spacer
import bokeh.models
from bokeh.models import ColumnDataSource,HoverTool,Range1d, Legend
from bokeh.models.glyphs import ImageURL
from bokeh.models.widgets import Button, CheckboxGroup, DataTable, Div, PreText, RadioGroup, RangeSlider, Select, Slider, TableColumn
from bokeh.models.formatters import NumeralTickFormatter
from bokeh.transform import factor_cmap
from bokeh import plotting



from collections import OrderedDict

import datetime

import gensim
from gensim import corpora
from gensim.test.utils import datapath

import NLP_fct

import nltk
from nltk import FreqDist
nltk.download('stopwords')

import numpy as np

import os
from os.path import dirname, join

import pandas as pd

import pyLDAvis.gensim

from shutil import copy

import timeit

from wordcloud import WordCloud

###############################################################################
#                               PARAMETRES                                    #
###############################################################################
APP_DIR         = os.path.dirname(__file__)
APP_DIR_PARENT  = os.path.dirname(APP_DIR)

# CHARGEMENT DES PARAMETRES FICHIERS ET VARIABLES #
###################################################
param_app_to_load   = pd.read_csv(os.path.join(APP_DIR_PARENT, 'topcapy_parameters_app.csv'), sep=',', encoding='utf-8')
param_app_to_load.set_index('parameter',inplace=True)

APP_FILE    = param_app_to_load.loc['file_name','parameter_value']
APP_VAR     = param_app_to_load.loc['var_name','parameter_value']
APP_GRADE   = param_app_to_load.loc['grade_name','parameter_value']
APP_MODEL   = param_app_to_load.loc['model_name','parameter_value']

copy(os.path.join(APP_DIR_PARENT, 'model/'+str(APP_MODEL)+'.html'), os.path.join(APP_DIR, 'static/'+str(APP_MODEL)+'.html'))



# CHARGEMENT DES PARAMETRES DE NOTATION #
#########################################
param_grade_to_load   = pd.read_csv(os.path.join(APP_DIR_PARENT, 'topcapy_parameters_grade.csv'), sep=',', encoding='utf-8')

list_grade = param_grade_to_load.columns.values.tolist()
dict_statut_note = {}
dict_statut_note['Tous'] = []
dict_statut_color = {}
for i in list_grade :
    dict_statut_note['Tous'] = dict_statut_note['Tous'] + param_grade_to_load.loc[0,i].split()
    dict_statut_color['Tous'] = 'blue'
    dict_statut_note[i]     = param_grade_to_load.loc[0,i].split()
    dict_statut_color[i]    = param_grade_to_load.loc[1,i]

liste_statuts = list(dict_statut_note.keys())



###############################################################################
#                           IMPORT DES DONNEES                                #
###############################################################################
# Import des données au format CSV
print('********** Import des données... **********')
file_to_import = os.path.join(APP_DIR_PARENT+str("/data/"), APP_FILE)
comments = pd.read_csv(file_to_import, sep=';', encoding='utf-8')
print('********** données importées ! **********')


###############################################################################
#                              PRE-TRAITEMENTS                                #
###############################################################################
list_mwe = ['turn over','parce que','prise en charge']
list_sw = ['celui','nan']
min_len_w = 2

# PRETRAITEMENT DES DONNÉES
print('********** Prétraitement des données... **********')
comments = NLP_fct.fct_pretraitement(comments,APP_VAR,min_len_w,list_mwe,list_sw)
print('********** prétraitement des données terminé ! **********')

# CONSERVATION DES LIGNES AVEC VERBATIM ET NOTE RENSEIGNÉES #
comments_vbt = comments[comments[APP_VAR].notnull() & comments[APP_GRADE].notnull()]

# RACINISATION
#comments['VbtTokenNoswStem'] = comments['VbtTokenNosw'].apply(lambda text : fct_stem(str(text)))



###############################################################################
#                            INTERFACE GRAPHIQUE                              #
###############################################################################




app_tabs_d = OrderedDict()

app_tabs_d["anexplo"] = {
    "title": "Exploration du corpus",
    "show": True,
    }

app_tabs_d["modellda"] = {
    "title": "Modélisation thématique",
    "show": True,
    }

app_tabs_d["antopics"] = {
    "title": "Analyse thématique",
    "show": True,
    }

app_tabs_d["anfeel"] = {
    "title": "Exploration sentiments / mots",
    "show": True,
    }

app_tabs_d["anfeelsent"] = {
    "title": "Exploration sentiments / phrases",
    "show": True,
    }


exec(open(os.path.join(APP_DIR, 'tab_anexplo.py')).read())
exec(open(os.path.join(APP_DIR, 'tab_modellda.py')).read())
exec(open(os.path.join(APP_DIR, 'tab_antopics.py')).read())
exec(open(os.path.join(APP_DIR, 'tab_anfeel.py')).read())
exec(open(os.path.join(APP_DIR, 'tab_anfeelsent.py')).read())


for tab_name, tab in app_tabs_d.items():
    tab_basename = "tab_" + tab_name
    app_tabs_d[tab_name]["panel"] = \
        bokeh.models.widgets.Panel(
            child=globals()[tab_basename + "_layout"],
            title=tab["title"])
        
app_tabs_list = [tab["panel"] for tab in app_tabs_d.values() \
                 if tab["show"]]
    
app_tabs = bokeh.models.widgets.Tabs(tabs=app_tabs_list)
bokeh.plotting.curdoc().add_root(app_tabs)        
#update(statut=liste_statuts[radiogroup_nps.active])

