#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Sep 10 14:33:57 2019

@author: Anne-Cécile GAY
"""



###############################################################################
#                                PARAMETRES                                   #                                    
###############################################################################

# Nombre minimal d'apparition du mot dans le corpus pour qu'il soit affiché
freq_min_word_to_show = 10

###############################################################################
#                             PRE TRAITEMENTS                                 #       
###############################################################################
list_statut = list(dict_statut_note.keys())
list_statut.remove('Tous')

if os.path.exists(datapath(os.path.join(APP_DIR_PARENT, 'metadata_vocab.csv'))):
    print('metadata_vocab déjà existant, pas de nouveaux calculs !')
    metadata_vocab = pd.read_csv(datapath(os.path.join(APP_DIR_PARENT, 'metadata_vocab.csv')), sep=';', encoding='utf-8')
    metadata_vocab.set_index('word',inplace=True,drop=False)

else :
    print('Fichier non existant : calculs en cours !')
    #LISTE DE VOCABULAIRE DU CORPUS : alimentation d'un dictionnaire de vocabulaire
    debut = timeit.default_timer()
    metadata_vocab = NLP_fct.fct_df_vocab(comments_vbt,'VbtTokenNosw')
    metadata_vocab.set_index('word', drop = False, inplace = True)
    duree = timeit.default_timer() - debut
    print('durée traitement alimentation du vocabulaire :')
    print(duree)
    
    # NOTE MOYENNE PAR MOT
    debut = timeit.default_timer()
    def fct_note_moy_word_comments(word): return NLP_fct.fct_note_moy_word(comments_vbt,APP_GRADE,word)
    metadata_vocab['mean_note']= metadata_vocab['word'].apply(fct_note_moy_word_comments)
    duree = timeit.default_timer() - debut
    print('durée traitement note moyenne par mot :')
    print(duree)
    
    # EFFECTIF DETRACTEURS, NEUTRES, PROMOTEURS
    for statut in dict_statut_note :
        debut = timeit.default_timer()
        print('début du calcul du statut par mot pour le statut : ')
        print(statut)
        def fct_temp(word) : return  NLP_fct.nb_statut_word(comments_vbt,APP_GRADE,word,statut,dict_statut_note)
        metadata_vocab[statut]= metadata_vocab['word'].apply(fct_temp)
        duree = timeit.default_timer() - debut
        print('fin du calcul du statut par mot pour le statut : ')
        print(statut)
        print('durée :')
        print(duree)
    
    metadata_vocab.to_csv(datapath(os.path.join(APP_DIR_PARENT, 'metadata_vocab.csv')), index = None, header=True, sep=';') 

  
# LISTE DES MOTS APPARAISSANT AU MOINS x FOIS
df_vocab_filtre = metadata_vocab.loc[metadata_vocab['occurrence']>=freq_min_word_to_show]

# MOT AVEC LA MEILLEURE NOTE
df_vocab_filtre.sort_values(by = 'mean_note',ascending = False, inplace=True)
bestword = df_vocab_filtre['word'].iloc[0]



list_vocab_filtre = metadata_vocab.loc[metadata_vocab['occurrence']>=freq_min_word_to_show]['word'].tolist()



###############################################################################
#                         INTERFACE GRAPHIQUE                                 #       
###############################################################################
anfeel_current_source = ColumnDataSource()

anfeel_current_source.data = df_vocab_filtre.to_dict('list')

def update_anfeel_selectword(word): 
    mean_note = df_vocab_filtre.loc[word]['mean_note']
    occurrence = df_vocab_filtre.loc[word]['occurrence']
    div_anfeel_notemoy.text = """
    <table style="border:1px solid white;" BGCOLOR=#0277bd width=50%>
        <tr>
        </tr>
        <tr>
            <td>
                <div align=center>                
                    <FONT size = 4 color=white>
                        <strong>
                            {}<br>
                        </strong>
                    </FONT>                
                    <FONT size = 7 color=white> 
                        <strong >
                           {:.1f}<br>
                        </strong>
                    </FONT>
                    <FONT size = 1 color=white>
                        <strong >
                            NOTE MOYENNE
                        </strong>
                    </FONT>    
                </div>
            </td>
        </tr>
        <tr>
        </tr>
    </table>
    """.format(word,mean_note)
    
    div_anfeel_occurrence.text = """
    <table style="border:1px solid white;" BGCOLOR=#0277bd width=50%>
        <tr>
        </tr>
        <tr>
            <td>
                <div align=center>
                    <FONT size = 4 color=white>
                        <strong>
                            {}<br>
                        </strong>
                    </FONT>                              
                    <FONT size = 7 color=white> 
                        <strong >
                           {}<br>
                        </strong>
                    </FONT>
                    <FONT size = 1 color=white>
                        <strong >
                            NOMBRE OCCURRENCES
                        </strong>
                    </FONT>    
                </div>
            </td>
        </tr>
        <tr>
        </tr>
    </table>
    """.format(word,occurrence)
    p_anfeel_rep_statut_word.y_range.factors = [word]
    
current_anfeel_selectword_examplespos = ColumnDataSource()  
 
def update_anfeel_selectword_examplespos(word):
    df_comments_word = comments_vbt.loc[comments_vbt["VbtTokenNosw"].str.contains("(\s|^)"+word+"(\s|$)",regex=True)]
    df_comments_word.sort_values(by = APP_GRADE,ascending = False, inplace=True)
    
    current_anfeel_selectword_examplespos.data = {
            'comments_pos'  : df_comments_word[APP_VAR],
            'note'          : df_comments_word[APP_GRADE]
        }
    
       
    
######################
#     GRAPHIQUES     #
######################

# GRAPHIQUE RÉPARTITION PROMOTEURS/NEUTRES/DETRACTEURS PAR MOT
p_anfeel_rep_statut_word = plotting.figure(y_range=bokeh.models.FactorRange(), plot_width=600, plot_height=100)
p_anfeel_rep_statut_word.axis.visible = False
p_anfeel_rep_statut_word.grid.visible = False
p_anfeel_rep_statut_word.outline_line_color = None

p_iter = p_anfeel_rep_statut_word.hbar_stack(list_statut, y='word', legend=[value(x) for x in list_statut],height=0.8, source=anfeel_current_source, color=[dict_statut_color[x] for x in list_statut])

for r in p_iter:
    statut = r.name
    hover = HoverTool(tooltips=[
        ("Eff. %s" % statut, "@%s" % statut)
    ], renderers=[r])
    p_anfeel_rep_statut_word.add_tools(hover)
    
p_anfeel_rep_statut_word.legend.location = "bottom_left"
p_anfeel_rep_statut_word.legend.orientation = "horizontal"



# WORDCLOUD MOTS POSITIFS
nb_top_mots_pos = 15

df_positive_words = df_vocab_filtre.sort_values(by = 'mean_note',ascending = False).iloc[0:nb_top_mots_pos,:]
positive_words = dict(zip(df_positive_words.word, df_positive_words.occurrence))
        
def random_color_func_pos(word=None, font_size=None, position=None, orientation=None, font_path=None, random_state=None):
#    h = int(360.0 * 60 / 255.0)
#    s = int(100.0 * 255.0 / 255.0)
#    l = int(100.0 * float(random_state.randint(60, 120)) / 255.0)
    h = 190
    s = int(100.0 * 255.0 / 255.0)
    l = int(100.0 * float(random_state.randint(60, 120)) / 255.0)
    return "hsl({}, {}%, {}%)".format(h, s, l)

wordcloud_pos = WordCloud(width=600, height=400, margin=0,prefer_horizontal=0.8,max_words=40, background_color='white',color_func=random_color_func_pos).generate_from_frequencies(positive_words)

wordcloud_pos.to_file(datapath(os.path.join(APP_DIR, 'static/wordcloud_pos.png')))
p_anfeel_wordcloud_pos = plotting.figure(x_range=(0,1), y_range=(0,1),plot_width=400, plot_height=250)
p_anfeel_wordcloud_pos.axis.visible = False
p_anfeel_wordcloud_pos.grid.visible = False
p_anfeel_wordcloud_pos.outline_line_color = None


url='/TOPCAPY_exploration/static/wordcloud_pos.png'
p_anfeel_wordcloud_pos.image_url(url='url', x=0, y=0, w=1, 
                          h=1,anchor="bottom_left",source=ColumnDataSource(data=dict(url=[url])))


# WORDCLOUD MOTS NEGATIFS
nb_top_mots_neg = 15
df_negative_words = df_vocab_filtre.sort_values(by = 'mean_note',ascending = True).iloc[0:nb_top_mots_neg,:]
negative_words = dict(zip(df_negative_words.word, df_negative_words.occurrence))

def random_color_func_neg(word=None, font_size=None, position=None, orientation=None, font_path=None, random_state=None):
    h = 30
    s = int(100.0 * 255.0 / 255.0)
    l = int(100.0 * float(random_state.randint(60, 120)) / 255.0)

    return "hsl({}, {}%, {}%)".format(h, s, l)

wordcloud_neg = WordCloud(width=600, height=400, margin=0,prefer_horizontal=0.8,max_words=40, background_color='white',color_func=random_color_func_neg).generate_from_frequencies(negative_words)
wordcloud_neg.to_file(datapath(os.path.join(APP_DIR, 'static/wordcloud_neg.png')))

p_anfeel_wordcloud_neg = plotting.figure(x_range=(0,1), y_range=(0,1),plot_width=400, plot_height=250)
p_anfeel_wordcloud_neg.axis.visible = False
p_anfeel_wordcloud_neg.grid.visible = False
p_anfeel_wordcloud_neg.outline_line_color = None

url_neg='/TOPCAPY_exploration/static/wordcloud_neg.png'
p_anfeel_wordcloud_neg.image_url(url='url', x=0, y=0, w=1, 
                          h=1,anchor="bottom_left",source=ColumnDataSource(data=dict(url=[url_neg])))


# TEXTE WORDCLOUD MOTS POSITIFS
div_anfeel_wordcloud_pos = Div()   
div_anfeel_wordcloud_pos.text = """
    <table style="border:1px solid white;" BGCOLOR=#0277bd width=100%>
        <tr>
        </tr>
        <tr>
            <td>
                <div align=center>                              
                    <FONT size = 2 color=white> 
                        <strong >
                           top {} des mots positifs <br>
                        </strong>
                    </FONT>
                </div>
            </td>
        </tr>
        <tr>
        </tr>
    </table>
    """.format(nb_top_mots_pos)
    
    
# TEXTE WORDCLOUD MOTS POSITIFS
div_anfeel_wordcloud_neg = Div()   
div_anfeel_wordcloud_neg.text = """
    <table style="border:1px solid white;" BGCOLOR=#0277bd width=100%>
        <tr>
        </tr>
        <tr>
            <td>
                <div align=center>                              
                    <FONT size = 2 color=white> 
                        <strong >
                           top {} des mots négatifs <br>
                        </strong>
                    </FONT>
                </div>
            </td>
        </tr>
        <tr>
        </tr>
    </table>
    """.format(nb_top_mots_neg)


# DATATABLE EXEMPLES COMMENTAIRES
columns = [
    TableColumn(field="comments_pos", title="Commentaires"),
    TableColumn(field="note", title="Note")
    ]
data_table_anfeel_selectword_examplespos = DataTable(source=current_anfeel_selectword_examplespos, columns=columns, width=600,height = 200,index_position=None,fit_columns=False)   

  
######################
# FONCTIONS CALLBACK #
######################
    
def callback_anfeel_selectwords(attr,old, new):
    update_anfeel_selectword(new)
    update_anfeel_selectword_examplespos(new)
    
    
    
######################
#      WIDGETS       # 
######################  
 
# WIDGET MENU DÉROULANT - sélection du mot à analyser 
select = Select(value=bestword, options=list_vocab_filtre,width=200)
select.on_change('value',callback_anfeel_selectwords)

# WIDGET TEXTE
div_anfeel_notemoy          = Div()    
div_anfeel_occurrence       = Div()   
div_anfeel_zoom_mot         = Div()   
div_anfeel_zoom_mot.text = """
    <table style="border:1px solid white;" BGCOLOR=#0277bd width=250%>
        <tr>
            <td>
                <div align=center>                
                    <FONT size = 2 color='white'>
                        <strong>
                            zoom par mot <br>
                        </strong>
                    </FONT>                  
                </div>
            </td>
        </tr>
    </table>
    """ 


######################
#   INITIALISATION   # 
######################

update_anfeel_selectword(bestword) 
update_anfeel_selectword_examplespos(bestword) 

    
######################
#      LAYOUT        # 
######################

layout_worcloud = row(Spacer(width=150, height=350,sizing_mode='scale_width'),column(div_anfeel_wordcloud_pos,p_anfeel_wordcloud_pos),Spacer(width=100, height=350,sizing_mode='scale_width'),column(div_anfeel_wordcloud_neg,p_anfeel_wordcloud_neg))

layout_word             =  row(column(div_anfeel_notemoy,div_anfeel_occurrence), column(p_anfeel_rep_statut_word,data_table_anfeel_selectword_examplespos))
layout_texte_zoom_mot   = row(Spacer(width=250, height=10,sizing_mode='scale_width'),div_anfeel_zoom_mot)
layout_zoom_mot         = row(select,layout_word)

tab_anfeel_layout = column(layout_worcloud,layout_texte_zoom_mot,layout_zoom_mot)