#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Aug 12 23:05:43 2019

@author: Anne-Cécile GAY
"""

from bokeh.core.properties import value
from bokeh.io import curdoc, show
from bokeh.layouts import layout,row, column, Spacer
import bokeh.models
from bokeh.models import ColumnDataSource,HoverTool,Range1d, Legend
from bokeh.models.glyphs import ImageURL
from bokeh.models.widgets import Button, CheckboxGroup, DataTable, Div, PreText, RadioGroup, RangeSlider, Select, Slider, TableColumn
from bokeh.models.formatters import NumeralTickFormatter
from bokeh import plotting

from collections import OrderedDict

import datetime

import gensim
from gensim import corpora
from gensim.test.utils import datapath

import NLP_fct

import nltk
from nltk import FreqDist
nltk.download('stopwords')

import numpy as np

import os
from os.path import dirname, join

import pandas as pd

import pyLDAvis.gensim

import timeit

from wordcloud import WordCloud



###############################################################################
#                               PARAMETRES                                    #
###############################################################################

APP_DIR         = os.path.dirname(__file__)
APP_DIR_PARENT  = os.path.dirname(os.path.dirname(__file__))

###################################################
# CHARGEMENT DES PARAMETRES FICHIERS ET VARIABLES #
###################################################
param_app_to_load   = pd.read_csv(os.path.join(APP_DIR_PARENT, 'topcapy_parameters_app.csv'), sep=',', encoding='utf-8')
param_app_to_load.set_index('parameter',inplace=True)
 
APP_FILE    = param_app_to_load.loc['file_name','parameter_value']
APP_VAR     = param_app_to_load.loc['var_name','parameter_value']
 
 
 
###############################################################################
#                           IMPORT DES DONNEES                                #
###############################################################################
# Import des données au format CSV
file_to_import = os.path.join(APP_DIR_PARENT+str("/data/"), APP_FILE)
comments = pd.read_csv(file_to_import, sep=';', encoding='utf-8')
 
 
 
###############################################################################
#                              PRE-TRAITEMENTS                                #
###############################################################################
list_mwe = ['turn over','parce que','prise en charge']
list_sw = ['celui','nan']
min_len_w = 2
 
comments = NLP_fct.fct_pretraitement(comments,APP_VAR,min_len_w,list_mwe,list_sw)
  
# CONSERVATION DES LIGNES AVEC VERBATIM RENSEIGNÉES #
comments_vbt = comments[comments[APP_VAR].notnull()]

# Messages log
msg_log = []
 
# Sablier
spinner_text = """
   <!-- https://www.w3schools.com/howto/howto_css_loader.asp -->
   <div class="loader">
   <style scoped>
   .loader {
       border: 16px solid #f3f3f3; /* Light grey */
       border-top: 16px solid #3498db; /* Blue */
       border-radius: 50%;
       width: 50px;
       height: 50px;
       animation: spin 2s linear infinite;
   }

   @keyframes spin {
       0% { transform: rotate(0deg); }
       100% { transform: rotate(360deg); }
   } 
   </style>
   </div>
   """

###############################################################################
#                               UPDATE                                        #
###############################################################################
def update_div(myText):
    msg_log.append('{}'.format(myText))
    m=''
    for msg in msg_log :
         m += '<li>{}</li>'.format(msg)
         div.text = '<ul>{}</ul>'.format(m)

def update_topcapymodel_modellda():
    div_loader.text=spinner_text
    # Création du dictionnaire
    num_iter = 0
    update_div("Préparation des données...")
    dictionary, corpus_bow = NLP_fct.init_dico_corpusbow(comments,'VbtTokenNoswStem')
    dictionary.save('dictionary.gensim')
  
    # Récupération choix sliders
    choix_nb_topics_start = slider_topcapymodel_select_nb_topics.value[0]
    choix_nb_topics_end = slider_topcapymodel_select_nb_topics.value[1]
    choix_nb_iters  = slider_topcapymodel_select_nb_iters.value
    update_div("Nombre de topics choisis par l'utilisateur : " + str(slider_topcapymodel_select_nb_topics.value))
    update_div("Nombre d'iterations choisies par l'utilisateur : " + str(slider_topcapymodel_select_nb_iters.value))
    
    # Lancement modélisation
    for i in range(choix_nb_topics_start,choix_nb_topics_end+1) :
        num_iter=num_iter+1
        update_div("*******************************************************")
        update_div("Modélisation n°"+str(num_iter)+" : "+str(i)+" topic(s) / "+str(choix_nb_iters)+" iteration(s)")
        update_div("*******************************************************")
        update_div("modélisation n°"+str(num_iter)+" en cours...")
        topics, ldamodel = NLP_fct.model_lda(dictionary, corpus_bow, i, choix_nb_iters)
        update_div("modélisation terminée !")
        # Sauvegarde du modèle
        update_div("sauvegarde du modèle n°"+str(num_iter)+" en cours...")
        date = datetime.datetime.now()
        modelname = "modellda_"+str(date.year)+str(date.month)+str(date.day)+"_"+str(date.hour)+str(date.minute)+str(date.second)
        ldamodel.save(os.path.join(APP_DIR_PARENT, 'model/'+str(modelname)+'.gensim'))
        update_div("modèle sauvegardé !")
        # Création graphique via LDAvis
        update_div("creation dataviz modèle n°"+str(num_iter)+" en cours...")
        lda_display = pyLDAvis.gensim.prepare(ldamodel, corpus_bow, dictionary, sort_topics=False)
        pyLDAvis.display(lda_display)
        pyLDAvis.save_html(lda_display,os.path.join(APP_DIR_PARENT, 'model/'+str(modelname)+'.html'))  
        update_div("dataviz terminée !")
        
    update_div("***************************************************")
    update_div("Toutes les modélisations demandées sont terminées !")
    update_div("***************************************************")
    
    div_loader.text=""


###############################################################################
#                             CALLBACKS                                       #
###############################################################################

def callback_topcapymodel_clicbutton_param():
    update_div("******************************************************************")
    update_div("*                                                                *")
    update_div("*                 LANCEMENT MODELISATIONS LDA                    *")
    update_div("*                                                                *")
    update_div("******************************************************************")
    update_topcapymodel_modellda()



###############################################################################
#                               WIDGETS                                       #
###############################################################################

# WIDGET SLIDER NOMBRE TOPICS
slider_topcapymodel_select_nb_topics = RangeSlider(title="Sélection nombre topics", start=2, end=30, value=(2,10), step=1)
#slider_modellda_select_nb_topics.on_change('value', callback_modellda_selectnbtopics)

# WIDGET SLIDER NOMBRE ITERATIONS
slider_topcapymodel_select_nb_iters = Slider(title="Sélection nombre itérations", start=1, end=100000, value=1, step=1)
#slider_modellda_select_nb_iters.on_change('value', callback_modellda_selectnbiters)

# WIDGET BUTTON NOUVELLE MODELISATION
button_topcapymodel_param_model = Button(label="lancement modélisations",width=100, button_type="primary")
button_topcapymodel_param_model.on_click(callback_topcapymodel_clicbutton_param)

# DIV TEXTE AVANCEMENT MODELISATION
div = Div(text='',width=400, height=50)

# WIDGET SABLIER
div_loader = Div(text='')



###############################################################################
#                                LAYOUT                                       #
###############################################################################
tab_topcapymodel_layout =row( column(slider_topcapymodel_select_nb_topics,slider_topcapymodel_select_nb_iters,button_topcapymodel_param_model,div_loader),div)
curdoc().add_root(tab_topcapymodel_layout)